#!/usr/bin/env node
"use strict";

exports.__esModule = true;
exports.testMatch = exports.getMatcher = exports.defaultInclude = exports.defaultExclude = exports.changeExt = void 0;

var _walk = _interopRequireDefault(require("walk"));

var _path = _interopRequireDefault(require("path"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _yargs = _interopRequireDefault(require("yargs"));

var _chalk = _interopRequireDefault(require("chalk"));

var _cosmiconfig = require("cosmiconfig");

var _lodash = _interopRequireDefault(require("lodash"));

var _micromatch = _interopRequireDefault(require("micromatch"));

var _core = require("@babel/core");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const explorer = (0, _cosmiconfig.cosmiconfig)('poi-transpile');

const {
  presets,
  plugins
} = require('./babel.config');
/* eslint-disable prefer-destructuring */


const argv = _yargs.default.usage('Usage: $0 [source] [options]').string('sm').alias('sm', 'source-map').describe('sm', 'saves sourcemap to .js.map file and/or inline. same as babel sourceMaps option, if provided with no value, it will default to true').boolean('replace').describe('replace', 'removes .es files').help('h').alias('h', 'help').argv;
/* eslint-enable prefer-destructuring */


const changeExt = (srcPath, ext) => {
  const srcDir = _path.default.dirname(srcPath);

  const srcBasename = _path.default.basename(srcPath, _path.default.extname(srcPath));

  return _path.default.join(srcDir, srcBasename + ext);
};

exports.changeExt = changeExt;
const defaultInclude = ['es', 'ts', 'tsx'].map(ext => `**/*.${ext}`);
exports.defaultInclude = defaultInclude;
const defaultExclude = ['**/*.d.ts'];
exports.defaultExclude = defaultExclude;

const getMatcher = (include, exclude) => include.concat(exclude.map(glob => `!${glob}`));

exports.getMatcher = getMatcher;

const testMatch = (target, include, exclude) => {
  return _micromatch.default.isMatch(target, include) && !_micromatch.default.isMatch(target, exclude);
};

exports.testMatch = testMatch;

const compileToJsAsync = async (appDir, replace, sm) => {
  let cosmicResult;

  try {
    cosmicResult = await explorer.search();
  } catch (e) {
    console.error('Cannot read the config file.');
  }

  const options = {
    followLinks: false,
    filters: ['node_modules', '.git']
  };

  const include = _lodash.default.toArray(_lodash.default.get(cosmicResult, ['config', 'include'], defaultInclude));

  const exclude = _lodash.default.toArray(_lodash.default.get(cosmicResult, ['config', 'exclude'], defaultExclude));

  return new Promise(resolve => {
    const tasks = [];

    _walk.default.walk(appDir, options).on('file', (root, fileStats, next) => {
      const match = testMatch(fileStats.name, include, exclude);

      if (match) {
        tasks.push(async () => {
          const srcPath = _path.default.join(root, fileStats.name);

          const codePath = changeExt(srcPath, '.js');
          const mapPath = changeExt(srcPath, '.js.map');
          let result;

          try {
            result = await (0, _core.transformFileAsync)(srcPath, {
              presets,
              plugins,
              babelrc: false,
              sourceMap: sm || true
            });
            console.log(_chalk.default.green(srcPath));
          } catch (e) {
            console.log(_chalk.default.red(`Error in processing ${srcPath}`));
            console.error(e.stack);
            process.exitCode = 1;
            return;
          }

          const {
            code,
            map
          } = result;
          await _fsExtra.default.writeFile(codePath, code);

          if (typeof sm !== 'undefined') {
            await _fsExtra.default.outputJson(mapPath, map);
          }

          if (replace) {
            await _fsExtra.default.remove(srcPath);
          }
        });
      }

      next();
    }).on('end', async () => {
      resolve(await Promise.all(tasks.map(f => f())));
    });
  });
};

const main = async () => {
  const source = _path.default.resolve(process.cwd(), argv._[0] || process.cwd());

  console.info('compiling', source);

  if (typeof argv.sm !== 'undefined') {
    console.info('Sourcemap generation enabled.');
  }

  if (argv.replace) {
    console.info('Replacing .es files.');
  }

  await compileToJsAsync(source, argv.replace, argv.sm);
};

if (require.main === module) {
  main();
}